/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include "dockbarwidget.h"
#include "ui_dockbarwidget.h"
#include "../libraries/mimetypexdg.h"
#include "../libraries/xfitman.h"
#include "dialogsettings.h"
#include "dquicklaunch/quicklaunchbutton.h"
#include "dquicklaunch/quicklaunchlayout.h"
#include "dquicklaunch/quicklaunchaction.h"
#include "dquicklaunch/dialogaddaction.h"
#include "../libraries/menugen.h"
#include "dquicklaunch/dactiontaskbar.h"
#include "dquicklaunch/xfitman.h"
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <QX11Info>

#include <QApplication>
#include <QDesktopWidget>
#include <QTimer>
#include <QDebug>
#include <QMouseEvent>
#include <QMenu>
#include <QFileSystemModel>
#include <QFileIconProvider>
#define EDGE_BOT 0     /*bottom edge*/
#define EDGE_TOP 1     /*top edge*/

#define TYPE_AUTO 0     /*type auto hide*/
#define TYPE_SPACE 1     /*type reserve space*/
#define TYPE_TOP 2     /*type on top*/
#define TYPE_BOT 3     /*type on bottom*/
#define MIMETYPE "ddock"
DockBarWidget::DockBarWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::DockBarWidget),
          m_minH(16),
          m_wmBorder(10)
{
          ui->setupUi(this);
           QApplication::setAttribute(Qt::AA_DontShowIconsInMenus ,false);
          setWindowFlags(Qt::FramelessWindowHint);
       setAttribute(Qt::WA_X11NetWmWindowTypeDock);
       setObjectName("elokab-dock");
         setWindowTitle("Elokab-Dock");
          this->setPalette(Qt::transparent);
          this->setAttribute(Qt::WA_TranslucentBackground,true);
          m_rootWindow = QX11Info::appRootWindow();
          setAcceptDrops(true);
          setAttribute(Qt::WA_AlwaysShowToolTips);

          MenuGen *gen =    new MenuGen(this->locale().name().section("_",0,0),this);
          if(gen->appFilesExist()==true){
                  //  qDebug()<<"file generated";
          }else{
                 //  qDebug()<<"file exist";
          }

          delete gen;

     connect(QApplication::desktop(), SIGNAL(workAreaResized(int)),
                  this, SLOT(   resizeDock()));
          timerShow=new QTimer;
          timerHide=new QTimer;
          timerShowHide=new QTimer;
          connect(timerShow,SIGNAL(timeout()) ,this,SLOT(moveToShow()));
          connect(timerHide,SIGNAL(timeout()) ,this,SLOT(moveToHide()));
     connect(timerShowHide,SIGNAL(timeout()) ,this,SLOT(getActinveWindowProperty()));

     m_settings=new elokabSettings("elokab","elokabsettings");
      connect(m_settings,SIGNAL(iconThemeChanged()) ,this,SLOT(loadIconThemes()));
      connect(m_settings,SIGNAL(ThemesChanged()) ,this,SLOT(loadSetting()));

      connect(m_settings,SIGNAL(widgetStyleChanged()) ,this,SLOT(crateStyle()));
      crateStyle();

      this->setMinimumWidth(60);

   //  QIcon::setThemeName("Macbuntu-Icons");
          m_layout = new QuickLaunchLayout(ui->widgetBgr);
//ui->horizontalLayoutAction->addLayout(m_layout);

          actSettings=new QAction(trUtf8("Dock settings"),this);
          connect(actSettings,SIGNAL(triggered()) ,this,SLOT(dockSettings()));

          actAddActions=new QAction(trUtf8("Add Lancher"),this);
          connect(actAddActions,SIGNAL(triggered()) ,this,SLOT(addLancher()));

          actQuit=new QAction(trUtf8("Quit"),this);
          connect(actQuit,SIGNAL(triggered()),qApp,SLOT(quit()));

          m_actAddToLancher=new QAction(QIcon::fromTheme("list-add"),trUtf8("Add to lancher"),this);
 connect(m_actAddToLancher,SIGNAL(triggered()),this,SLOT(addAppTaskToLancher()));


 setContextMenuPolicy(Qt::CustomContextMenu);
 connect( this,SIGNAL(customContextMenuRequested(QPoint)) ,this,SLOT(customContextMenu(QPoint)));

  loadSetting();

 addQuickLaunch();
loadIconThemes();
}

DockBarWidget::~DockBarWidget()
{
    delete ui;
}

void DockBarWidget::crateStyle()
{
    QString curentStyle=m_settings->loadStyleWidget();
  QApplication::setStyle( QStyleFactory::create(curentStyle)) ;
}
void DockBarWidget::show()
{
          QWidget::show();
          xfitMan().moveWindowToDesktop(this->winId(), -1);
         resizeDock();
}
 void DockBarWidget::resizeEvent ( QResizeEvent * /*event*/ )
 {
     QRect screen = QApplication::desktop()->screenGeometry(-1);
     if(this->width()>=screen.width()){
        m_minH=m_minH-16;
         m_layout->setSizeH(m_minH);
         foreach (QuickLaunchButton *btnQ,m_buttonsQuick){
                   btnQ->setIconSize(QSize(m_minH,m_minH));
         }
         foreach (DActionTaskbar *btnT,m_buttonsTask){
                   btnT->setIconSize(QSize(m_minH,m_minH));
         }
         resizeDock();
     }
this->move((screen.width()/2)-(this->width()/2),this->pos().y());
//       repaint();        //
 }

 void DockBarWidget::enterEvent(QEvent *)
 {

           if(m_positionType==TYPE_BOT){
           //        setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint);

                     this->activateWindow();
                        this->raise();
                       this->show();
           }


           if(m_positionType==TYPE_AUTO){
                     timerShowHide->stop();
                     mouseEntered=true;
                     timerShow->singleShot(500, timerShow, SLOT(start()));
                     timerShow->setInterval(10);
           }
 }

 void DockBarWidget::leaveEvent(QEvent *)
 {
           if(m_positionType==TYPE_BOT){
                     if(QCursor::pos().x()<this->pos().x()||
                      QCursor::pos().y()<this->pos().y() ||
                       QCursor::pos().y()>this->pos().y()+this->height()||
                        QCursor::pos().x()>this->pos().x()+this->width()){
  //  setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnBottomHint);
         this->setVisible(true);
      this->lower();
                     }
           }
           if(m_positionType==TYPE_AUTO){
                     timerShowHide->start(1000);
                     mouseEntered=false;
//                     timerHide->singleShot(1000, timerHide, SLOT(start()));
//                     timerHide->setInterval(10);
           }
 }


 void DockBarWidget::moveToHide()
 {
           timerShow->stop();
             QRect screen = QApplication::desktop()->screenGeometry(-1);
             if(mouseEntered==true){
                       timerHide->stop();
                       return;
             }
//***************************************************************
             if(m_screenEdge==EDGE_BOT){
                       if(this->pos().y()>=screen.height()-2){
                                 timerHide->stop();
                                 return;
                       }
                       this->move(this->pos().x(),this->pos().y()+2);
             }else{
                       if(this->pos().y()<=-(this->height()-2)){
                                 timerHide->stop();
                                 return;
                       }
                       this->move(this->pos().x(),this->pos().y()-2);
             }
 }
 void DockBarWidget::moveToShow()
 {
           timerHide->stop();
           if(mouseEntered==false){
                     timerShow->stop();
                     return;
           }
           QRect screen = QApplication::desktop()->screenGeometry(-1);
           if(m_screenEdge==EDGE_BOT){
                     if(this->pos().y()<=screen.height()-this->height()){
                               timerShow->stop();
                               return;
                     }
                     this->move(this->pos().x(),this->pos().y()-2);
           }else{
                     if(this->pos().y()>=0){
                               timerShow->stop();
                               return;
                     }
                     this->move(this->pos().x(),this->pos().y()+2);
           }
 }
void DockBarWidget::resizeDock()
{

          QRect screen = QApplication::desktop()->screenGeometry(-1);
          resize(this->sizeHint().width(),this->sizeHint().height());
          int left=(screen.width()/2)-(this->width()/2);
          if(m_screenEdge==EDGE_BOT){

                    int top=screen.height()-this->height();
                    move(left,top);
          }else{
                    move(left,0);
          }

}
void DockBarWidget::loadSetting()
{

    QSettings setting("elokab","settings");

    setting.beginGroup("Dock-Style");
    m_positionType=setting.value("position",TYPE_AUTO).toInt();
    m_screenEdge=setting.value("screenEdge",EDGE_BOT).toInt();
    m_minH=setting.value("iconSize",48).toInt();
    m_animmationStyle=setting.value("animmationStyle",0).toInt();
   // QString style=setting.value("styleSheet",setStyleDefault()).toString();

    int bleft=(setting.value("borderLeft",5).toInt());
    int bright=(setting.value("borderRight",5).toInt());
    int btop=(setting.value("borderTop",5).toInt());
    int bbot=(setting.value("borderBot",5).toInt());
    setting.endGroup();
    update();
    repaint();
    m_layout->setContentsMargins(bleft,btop,bright,bbot);
    m_layout->setSizeH(m_minH);
    foreach (QuickLaunchButton *btnQ,m_buttonsQuick){
        btnQ->setIconSize(QSize(m_minH,m_minH));
        btnQ->setAnimation(m_animmationStyle);
    }
    foreach (DActionTaskbar *btnT,m_buttonsTask){
        btnT->setIconSize(QSize(m_minH,m_minH));
    }
    //********************************

   // ui->widgetBgr->setStyleSheet(style);
    QString qssFile=QDir::homePath()+"/.config/elokab/dockTheme.css";

     if(!QFile::exists(qssFile))
       qssFile= ":/icons/theme.css";

     QFile file;
   file.setFileName(qssFile);
   if(file.open( QFile::ReadOnly)){
       QTextStream textStream(&file);
       textStream.setCodec(QTextCodec::codecForLocale());
       QString line;
       line = textStream.readAll();//premier line


//              QFile qss(qssFile);
//                          qss.open(QFile::ReadOnly);
    ui->widgetBgr->setStyleSheet(line);
    file.close();
   }
    //*************************
    timerShowHide->stop();
    resizeDock();

    //********************************

    switch (m_positionType)
    {
    case TYPE_AUTO:              /* auto hide */

        setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint);
        timerShowHide->start(1000);
        m_autoHide=true;
        reserveSpace(false);
        break;
    case TYPE_SPACE:              /* reserve space */

        setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint);
        m_autoHide=false;
        reserveSpace(true);
        break;
    case TYPE_TOP:              /* on top */
        reserveSpace(false);
        m_autoHide=false;
        setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnTopHint);
        break;
    case TYPE_BOT:              /* on bottom */
        reserveSpace(false);
        m_autoHide=false;
        setWindowFlags(Qt::FramelessWindowHint|Qt::WindowStaysOnBottomHint);
        break;
    }
    //******************************
    this->show();

    QApplication::setLayoutDirection(Qt::LeftToRight);
    resizeDock();

}
void DockBarWidget::loadIconThemes()
{

  //  m_settings->loadIconThems();

  QString icnThem=m_settings->loadIconThems();
 //  qDebug()<<"m_settings   :   "<<icnThem;
  QIcon::setThemeName(icnThem);
  QIcon icn1=QIcon::fromTheme("configure",QIcon::fromTheme("document-properties"));
  QIcon icn2=QIcon::fromTheme("list-add",QIcon::fromTheme("add"));
  QIcon icn3=QIcon::fromTheme("application-exit",QIcon::fromTheme("exit"));

  actSettings->setIcon(icn1);
  actAddActions->setIcon(icn2);
  actQuit->setIcon(icn3);
  foreach (QuickLaunchButton *btnQ,m_buttonsQuick){
         btnQ->setAnimation(m_animmationStyle);
  }

}

void DockBarWidget::reserveSpace(bool arg)
{
          XfitMan xf = xfitMan();
          Window wid =this->winId();
          QRect screen = QApplication::desktop()->screenGeometry(0);
          QRect rect;
          int mScreenNum= QApplication::desktop()->primaryScreen();
          QRect desktop = QApplication::desktop()->screen(mScreenNum)->geometry();
          rect.moveLeft(screen.left());
          //  rect.moveTop(screen.top());
          if (m_screenEdge == EDGE_BOT){

                    rect = this->rect();
                    rect.moveBottom(screen.bottom());
          } else {
                    rect = this->rect();
                    rect.moveTop(screen.top());


          }
          this->setGeometry(rect);

          if (arg==false){
                    xf.setStrut(wid,  0, 0, 0, 0,
                                /* Left   */   0, 0,
                                /* Right  */   0, 0,
                                /* Top    */   0, 0,
                                /* Bottom */   rect.left(), rect.right()
                                );
                    return;
          }




          if (m_screenEdge == EDGE_BOT){
                    xf.setStrut(wid,  0, 0, 0, desktop.height() - rect.y(),
                                /* Left   */   0, 0,
                                /* Right  */   0, 0,
                                /* Top    */   0, 0,
                                /* Bottom */   rect.left(), rect.right()
                                );
          }else{
                    xf.setStrut(wid, 0, 0, rect.bottom(), 0,
                                /* Left   */   0, 0,
                                /* Right  */   0, 0,
                                /* Top    */   rect.left(), rect.right(),
                                /* Bottom */   0, 0
                                );

          }

          this->setGeometry(rect);
       //   m_rootWindow = QX11Info::appRootWindow();
}


void DockBarWidget::dockSettings()
{
    QRect screen = QApplication::desktop()->screenGeometry(-1);

    DialogSettings *dlg=new DialogSettings;
    connect(dlg,SIGNAL(accepteChange()) ,this,SLOT(loadSetting()));
    dlg->move(screen.width()/2-dlg->width()/2,screen.height()/2-dlg->height()/2);
    dlg->show();
}

//*****************************QuickLaunch************************
void DockBarWidget::addQuickLaunch()

{

    QSettings setting("elokab","dockQuickLanche") ;
    int count = setting.beginReadArray("QuickApps");
    if(count==0){
        addButton(new QuickLaunchAction( "/usr/share/applications/firefox.desktop", this));
        addButton(new QuickLaunchAction( "/usr/share/applications/libreoffice-writer.desktop", this));
        addButton(new QuickLaunchAction("x-terminal", "x-terminal-emulator","terminal", this));
        return;
    }

    QString desktop;
    QString file;
    QString execname;
    QString exec;
    QString icon;
    for (int i = 0; i < count; ++i){

        setting.setArrayIndex(i);
        desktop = setting.value("desktop", "").toString();
        file = setting.value("file", "").toString();

        if (! file.isEmpty()){
            //addButton(new QuickLaunchAction(file, this));
        }else{
            execname = setting.value("name", "").toString();
            exec = setting.value("exec", "").toString();
            icon = setting.value("icon", "").toString();
//            if (icon.isEmpty()) addButton(new QuickLaunchAction( exec, this));
//            else addButton(new QuickLaunchAction(execname, exec,icon, this));
 //           qDebug()<<"icontaskbar--------------:= "+icon;
addButton(new QuickLaunchAction(execname, exec,icon, this));  }
    } // for

    setting.endArray();
}

void DockBarWidget::customContextMenu(QPoint)
{
    QMenu menu;


    menu.addAction(actAddActions);
    menu.addAction(actSettings);
    menu.addSeparator();
    menu.addAction(actQuit);
    menu.exec(this->cursor().pos());
}
//*****************************QuickLaunch************************
int DockBarWidget::indexOfButton(QuickLaunchButton* button) const
{
          return m_layout->indexOf(button);
}
//*****************************QuickLaunch************************
int DockBarWidget::countOfButtons() const
{
          return m_layout->count();
}
//*****************************QuickLaunch************************
void DockBarWidget::addButton(QuickLaunchAction* action)
{

    int newIndex = 0;
    QList<int> keys = m_buttonsQuick.uniqueKeys();
    qSort(keys);
    foreach (int index, keys)
    {
        if (newIndex != index)
            break;
        ++newIndex;
    }

    QuickLaunchButton* btn = new QuickLaunchButton(QSize(m_minH,m_minH),newIndex, action, this);
    btn->setIconSize(QSize(m_minH,m_minH));
    m_layout->addWidget(btn);
    m_buttonsQuick[newIndex] = btn;
    btn->setAnimation(m_animmationStyle);
    connect(btn, SIGNAL(buttonDeleted(int)), this, SLOT(buttonDeleted(int)));
    connect(btn, SIGNAL(settingsChanged()), this, SLOT(saveSettings()));

    connect(btn, SIGNAL(switchButtons(int,int)), this, SLOT(switchButtons(int,int)));


}
//*****************************QuickLaunch************************
void DockBarWidget::dragEnterEvent(QDragEnterEvent *e)

{
    if (e->mimeData()->hasFormat(MIMETYPE))
    {
        e->acceptProposedAction();
        return;
    }
          // Getting URL from Desktp File...
          if (e->mimeData()->hasUrls())
          {
                    e->acceptProposedAction();
                    return;
          }

          if (e->source() && e->source()->parent() == this)
          {
                    e->acceptProposedAction();
          }
}
//*****************************QuickLaunch************************
void DockBarWidget::dropEvent(QDropEvent *e)
{
          const QMimeData *mime = e->mimeData();

          QList<QUrl> duplicates;

          foreach (QUrl url, mime->urls())
          {
              if (duplicates.contains(url))
                  continue;
              else
                  duplicates << url;

              QString fileName(url.toLocalFile());

           //   qDebug()<<"droped filename-------"<<fileName;
              addButton(new QuickLaunchAction(fileName, this));

          }
          saveSettings();
}
//*****************************QuickLaunch************************

void DockBarWidget::buttonDeleted(int id)
{
          QuickLaunchButton *b = m_buttonsQuick[id];
          m_layout->removeWidget(b);
          m_buttonsQuick.remove(id);
          b->deleteLater();
          saveSettings();
}

//*****************************QuickLaunch************************
void DockBarWidget::saveSettings()
{

          QSettings settings("elokab","dockQuickLanche") ;
          //    settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
          settings.remove("QuickApps");
          settings.beginWriteArray("QuickApps");
          int i = 0;

          foreach(QuickLaunchButton *b, m_layout->buttons())
          {
                    settings.setArrayIndex(i);

                    QHashIterator<QString,QString> it(b->settingsMap());
                    while (it.hasNext())
                    {
                              it.next();
                              settings.setValue(it.key(), it.value());
                    }

                    ++i;
          }

          settings.endArray();
}
//*****************************QuickLaunch************************
void DockBarWidget::addLancher()
{
    QRect screen = QApplication::desktop()->screenGeometry(-1);
    DialogAddAction *dlg=new DialogAddAction;
    connect(dlg,SIGNAL(accepteAdd(QString)) ,this,SLOT(addButton(QString)));
    dlg->move(screen.width()/2-dlg->width()/2,screen.height()/2-dlg->height()/2);
    dlg->show();

}
void DockBarWidget::addButton(QString txt)
{
        //  qDebug()<<"recever text ;"<<txt;
          addButton(new QuickLaunchAction(txt.section("|",0,0), txt.section("|",1,1), txt.section("|",2,2), this));
          saveSettings();
}

void DockBarWidget::switchButtons(int first, int second)
{

          QuickLaunchButton *w1 = m_buttonsQuick[first];
          QuickLaunchButton *w2 = m_buttonsQuick[second];
          m_layout->swapButtons(w1, w2);
          saveSettings();

}
//*******************Taskbar**************************
void DockBarWidget::handlePropertyNotify(XPropertyEvent* event)
{

    if (event->window == m_rootWindow)
    {
        XfitMan xf = xfitMan();
        // Windows list changed ...............................

        if (event->atom == XfitMan::atom("_NET_CLIENT_LIST"))
        {
            //qDebug()<<"getClientList : "<<  xf.getClientList();
            refreshTaskList();
            return;
        }

        // Activate window ....................................
        if (event->atom == XfitMan::atom("_NET_ACTIVE_WINDOW"))
        {

            activeWindowChanged();
            return;
        }

        // Desktop switch .....................................
        if (event->atom == XfitMan::atom("_NET_CURRENT_DESKTOP"))
        {

            return;
        }
    }
    if (event->atom == 292)
    {


        activeWindowChanged();
        //                refreshTaskList();
        return;
    }


}
//*******************Taskbar**************************
void DockBarWidget::refreshTaskList()
{
//    foreach (QuickLaunchButton *btn,m_buttonsQuick){

//        btn->setVisible(true);

//    }
    XfitMan xf = xfitMan();
    //              قائمة مؤقة بجميع معرفات النوافذ الحاضرة
    QList<Window> listWindow = xf.getClientList();

    //              قائمة بجميع معرفات النولفذ والازرار
    QMutableHashIterator<Window, DActionTaskbar*> i(m_buttonsTask);
    //                  حذف جميع الازرار ومعرفاتها السابقة
    while (i.hasNext())
    {
        i.next();

        //                    delete i.value();
        //                    i.remove();
        int r = listWindow.removeAll(i.key());
        if (!r)
        {

            foreach (QuickLaunchButton *btn,m_buttonsQuick){
               if(btn->data().contains("/") && btn->data().contains(i.value()->windowTitle()))
                    btn->setVisible(true);

               else if(btn->data()==i.value()->windowTitle())
                    btn->setVisible(true);
            }

            m_layout->removeWidget(i.value());
            delete i.value();
            i.remove();

        }
    }

    foreach (Window wnd, listWindow)
    {


        if (xf.acceptWindow(wnd))
        {
            //            انشاء زر جديد للنوافذ الحاضرة
            DActionTaskbar *btn=new DActionTaskbar(QSize(m_minH,m_minH),wnd,this);
           QString appname=btn->appName();
           btn->setObjectName("btnTask");
            //
           btn->setWindowTitle(appname);
           //
           QAction *actAddToLancher=new QAction(QIcon::fromTheme("list-add"),trUtf8("Add to lancher"),btn);
           connect(actAddToLancher,SIGNAL(triggered()),this,SLOT(addAppTaskToLancher()));
QList<QAction *>listact;

QAction *act=new QAction(btn);
act->setSeparator(true);
listact<<(actAddActions);
listact<<(actSettings);
listact<<act;
listact<<actQuit;
           btn->setParentAction(actAddToLancher);
           btn->setParentActions(listact);

            //            اضافة ازر للقائمة
            m_buttonsTask.insert(wnd, btn);

            //        اضافة الزر للبنال
            m_layout->addWidget(btn);


            foreach (QuickLaunchButton *btnk,m_buttonsQuick){

                if(btnk->data().contains("/") && btnk->data().contains(appname))
                    btnk->setVisible(false);

                else if(btnk->data()==appname)
                        btnk->setVisible(false);

            }
        }
    }



    activeWindowChanged();

}

void DockBarWidget::addAppTaskToLancher()
{
    //add
    QAction *action = qobject_cast<QAction *>(sender());
    //qDebug()<<"set  m_actAddToLancher data :"<<action->parent();
    DActionTaskbar *btn= qobject_cast<DActionTaskbar *>(action->parent());
    QString name=btn->appName();
    QString classename=btn->classeName();


QString missage=tr("this application exist");
    foreach (QuickLaunchButton *btnQuik,m_buttonsQuick){

        if(btnQuik->data().contains("/") && btnQuik->data().contains(name)){
            QMessageBox::information(0,"",missage);
            return;
        }
        else if(btnQuik->data()==name||btnQuik->data()==classename){
            QMessageBox::information(0,"",missage);
            return;

        }

    }
  QString appToAdd= MimeTypeXdg::getinfoAppFromDesktopApp(name,classename);
    if(appToAdd==""){
        QMessageBox::information(0,"",tr("this application is not xdg desktop valid"));
    return;
    }
    addButton(new QuickLaunchAction(appToAdd.section("|",0,0),
                                    appToAdd.section("|",1,1),
                                    appToAdd.section("|",2,2), this));


    saveSettings();
}

//*******************Taskbar**************************
void DockBarWidget::activeWindowChanged()
{


          foreach (DActionTaskbar *btn,m_buttonsTask){

                    btn->setText(xfitMan().getWindowTitle(m_buttonsTask.key(btn)));
                    btn->setChecked(false);
                    btn->setIconSize(QSize(m_minH,m_minH));


          }
          //        نزع التفعيل من كل الازرار
          QList<DActionTaskbar *> allButtons = this->findChildren<DActionTaskbar *>();

          //        البحث عن النافذة المفعلة
          Window window = xfitMan().getActiveAppWindow();
       //   qDebug()<<"window activated   "<<window;
          //         التاكد من وجودها مع الازرا
          DActionTaskbar* toolbtn = buttonByWindow(window);
          //       تفيل زر النافذة المفعلة اذا كان موجودا
          if (toolbtn){
                    toolbtn->setChecked(true);


          }else{ //        نزع التفعيل من كل الازرار
                    foreach (DActionTaskbar *btn, allButtons)
                    {
                              btn->setChecked(false);

                    }
          }

//           if(m_screenEdge==EDGE_TOP){

//               getwmBoreder(EDGE_TOP);
//           }else{
//                    getwmBoreder(EDGE_BOT);
//           }
getwmBoreder();

}


//*******************Taskbar**************************
DActionTaskbar* DockBarWidget::buttonByWindow(Window window) const
{
          if (m_buttonsTask.contains(window))
                    return m_buttonsTask.value(window);
          return 0;
}


//*******************Taskbar**************************
bool DockBarWidget::windowOnActiveDesktop(Window window) const
{

          XfitMan xf = xfitMan();
          int desktop = xf.getWindowDesktop(window);
          if (desktop == -1) // Show on all desktops
                    return true;

          if (desktop == xf.getActiveDesktop())
                    return true;

          return false;
}

//*******************Taskbar**************************
void  DockBarWidget::x11EventFilter(XEvent* event)
{

          switch (event->type)
          {
          case PropertyNotify:

                    handlePropertyNotify(&event->xproperty);

                    break;

#if 0
          case MotionNotify:
                    break;

          default:
          {
                 //   qDebug() << "** XEvent ************************";

          }
#endif

          }
}


void DockBarWidget::getActinveWindowProperty()
{
          QString wX,wY,wW,wH;
  QRect screen = QApplication::desktop()->screenGeometry(-1);
          Window window = xfitMan().getActiveAppWindow();

          if (window == 0||window==this->winId()){
              mouseEntered=true;
              timerHide->stop();
              timerShow->start(10);
                    return ;
          }
//qDebug()<<window;
          QProcess process;
          process.start(QString("xwininfo -id %1  -stats").arg(window));
          process.waitForFinished();

          QTextStream inh(process.readAll());
          while ( !inh.atEnd() )
          {
                    QString heightline = inh.readLine();
                    if(heightline.contains( "Height:"))
                    {

                              wH= heightline.section(":",1,1).trimmed();
                    }
                    //                      if(heightline.contains( "Width:"))
                    //                      {

                    //                               wW= heightline.section(":",1,1).trimmed();
                    //                      }
                    //                      if(heightline.contains( "Absolute upper-left X:"))
                    //                      {

                    //                              wX= heightline.section(":",1,1).trimmed();
                    //                      }
                    if(heightline.contains( "Absolute upper-left Y:"))
                    {

                              wY=heightline.section(":",1,1).trimmed();
                    }
          }

     //    qDebug()<<"top corner : "<<wY.toInt()+wH.toInt();
      //  qDebug()<<"this corner : "<<this->frameGeometry().height();
          if(m_screenEdge==EDGE_BOT){
                    if(wY.toInt()+wH.toInt()>screen.height()-this->frameGeometry().height()-m_wmBorder){
                              mouseEntered=false;
                           timerShow->stop();
                              timerHide->start(10);
                    }else{
                              mouseEntered=true;
                              timerHide->stop();
                              timerShow->start(10);
                    }
          }else{
                    if(wY.toInt()<this->frameGeometry().height()+m_wmBorder){
                              mouseEntered=false;
                              timerHide->start(10);
                    }else{
                              mouseEntered=true;
                              timerShow->start(10);
                    }
          }
}
void DockBarWidget::getwmBoreder()
{

    Window window = xfitMan().getActiveAppWindow();
    if (window == 0||window==this->winId())
        return;
    QProcess process;
    process.start(QString("xwininfo -id %1  -wm").arg(window));
    process.waitForFinished();
    QTextStream inh(process.readAll());
    while ( !inh.atEnd() )
    {
        QString heightline = inh.readLine();
        if(heightline.contains( "Frame extents:"))
        {
                if(m_screenEdge==EDGE_TOP){
                        m_wmBorder=heightline.section(",",2,2).trimmed().toInt()+1;
                }else{
                        m_wmBorder=heightline.section(",",3,3).trimmed().toInt()+1;
                }

        }
    }
    qDebug()<<m_wmBorder;
}
