/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include "dactiontaskbar.h"
#include "../libraries/mimetypexdg.h"
#include "xfitman.h"
#include <QtGui/QContextMenuEvent>
#include<QWidget>
#include<QToolButton>
DActionTaskbar::DActionTaskbar(QSize size,const Window window, QWidget *parent) :
          QToolButton(parent),
          m_Window(window)
{
          this->setCheckable(true);
          //this->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

setIconSize(size);
 //setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
          getText();
          getIcon();
          this->setMinimumWidth(16);
}


void DActionTaskbar::getIcon()
{
    QIcon icon;

    //from appname
    icon=QIcon::fromTheme(appName());
    if(!icon.isNull()){
        QPixmap pixmap = icon.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
        setIcon(QIcon(pixmap));
       // qDebug()<<"getIcon from appname-------------";
        return;
    }
    //from classe name
    icon=QIcon::fromTheme(classeName().toLower());
    if(!icon.isNull()){
        QPixmap pixmap = icon.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
        setIcon(QIcon(pixmap));
       // qDebug()<<"getIcon from classename-------------";
        return;
    }
     //from xdgappname
    QString xdgicon=MimeTypeXdg::getIconFromDesktopApp(appName());
    // qDebug()<<"getIconFromDesktopAppappName xdgicon------------"<<xdgicon;
    icon=QIcon::fromTheme(xdgicon);
    if(!icon.isNull()){
        QPixmap pixmap = icon.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
        setIcon(QIcon(pixmap));
         //qDebug()<<"getIconFromDesktopAppappName------------";
        return;
    }
    //from xdgclassename
    xdgicon=MimeTypeXdg::getIconFromDesktopApp(classeName());
   icon=QIcon::fromTheme(xdgicon);
   if(!icon.isNull()){
       QPixmap pixmap = icon.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
       setIcon(QIcon(pixmap));
      //  qDebug()<<"getIconFromDesktopAppclasseName-------------";
       return;
   }

    QPixmap pix ;

          if (xfitMan().getClientIcon(m_Window, pix))
              setIcon(QIcon(pix.scaled(128,128,Qt::IgnoreAspectRatio,Qt::SmoothTransformation)));

          else
              setIcon(QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")).pixmap(128));
}

void DActionTaskbar::getText()
{
          QString title = xfitMan().getWindowTitle(m_Window);

                               setText(title.replace("&", "&&"));
          setToolTip(title);
          //     qDebug()<<"title-------------------------------------"<<title;
}
QString DActionTaskbar::appName()
{
 //    qDebug()<<"getApplicationName : "<<   xfitMan().getApplicationName(m_Window);

    return  xfitMan().getApplicationName(m_Window);

}
QString DActionTaskbar::classeName()
{
  //   qDebug()<<"getApplicationClasseName : "<<   xfitMan().getApplicationClasseName(m_Window);

    return  xfitMan().getApplicationClasseName(m_Window);

}
void DActionTaskbar::contextMenuEvent(QContextMenuEvent* event)
{
          XfitMan xf = xfitMan();

          WindowAllowedActions allow = xf.getAllowedActions(m_Window);
          WindowState state = xf.getWindowState(m_Window);



          QMenu menu(trUtf8("Application"));
          QAction* a;



          // ** قوائم اسطح المكتب **************************
          int deskNum = xf.getNumDesktop();
          if (deskNum > 1)
          {
                    int winDesk = xf.getWindowDesktop(m_Window);
                    QMenu* deskMenu = menu.addMenu(tr("To &Desktop"));

                    a = deskMenu->addAction(tr("&All Desktops"));
                    a->setData(-1);
                    a->setEnabled(winDesk != -1);
                    connect(a, SIGNAL(triggered(bool)), this, SLOT(moveWindowToDesktop()));
                    deskMenu->addSeparator();

                    for (int i=0; i<deskNum; ++i)
                    {
                              a = deskMenu->addAction(tr("Desktop &%1").arg(i+1));
                              a->setData(i);
                              a->setEnabled( i != winDesk );
                              connect(a, SIGNAL(triggered(bool)), this, SLOT(moveWindowToDesktop()));
                    }

                    int curDesk = xf.getActiveDesktop();
                    a = menu.addAction(tr("&To Current Desktop"));
                    a->setData(curDesk);
                    a->setEnabled( curDesk != winDesk );
                    connect(a, SIGNAL(triggered(bool)), this, SLOT(moveWindowToDesktop()));
          }

          // **قوائم اوامر النوافذ ****************************
          menu.addSeparator();

          a = menu.addAction(QIcon(style()->standardPixmap(QStyle::SP_TitleBarMaxButton)),tr("Ma&ximize"));
          a->setEnabled((allow.MaximizeHoriz && allow.MaximizeVert) &&
                        (!state.MaximizedHoriz || !state.MaximizedVert /*|| state.Hidden*/));
          a->setData(XfitMan::MaximizeBoth);
          connect(a, SIGNAL(triggered(bool)), this, SLOT(maximizeWindow()));

          if (event->modifiers() & Qt::ShiftModifier)
          {

                    a = menu.addAction(QIcon(style()->standardPixmap(QStyle::SP_TitleBarMaxButton)),tr("Maximize vertically"));
                    a->setEnabled((allow.MaximizeVert) && (!state.MaximizedVert || state.Hidden));
                    a->setData(XfitMan::MaximizeVert);
                    connect(a, SIGNAL(triggered(bool)), this, SLOT(maximizeWindow()));

                    a = menu.addAction(QIcon(style()->standardPixmap(QStyle::SP_TitleBarMaxButton)),tr("Maximize horizontally"));
                    a->setEnabled((allow.MaximizeHoriz) && (!state.MaximizedHoriz || state.Hidden));
                    a->setData(XfitMan::MaximizeHoriz);
                    connect(a, SIGNAL(triggered(bool)), this, SLOT(maximizeWindow()));
          }

          a = menu.addAction(QIcon(style()->standardPixmap(QStyle::SP_TitleBarNormalButton)),tr("&Restore"));
          a->setEnabled(state.Hidden || state.MaximizedHoriz || state.MaximizedVert);
          connect(a, SIGNAL(triggered(bool)), this, SLOT(unMaximizeWindow()));

          a = menu.addAction(QIcon(style()->standardPixmap(QStyle::SP_TitleBarMinButton)),tr("Mi&nimize"));
          a->setEnabled(allow.Minimize && !state.Hidden);
          connect(a, SIGNAL(triggered(bool)), this, SLOT(minimizeWindow()));

          if (state.Shaded)
          {
                    a = menu.addAction(QIcon(style()->standardPixmap(QStyle::SP_TitleBarUnshadeButton)),tr("Roll down"));
                    a->setEnabled(allow.Shade && !state.Hidden);
                    connect(a, SIGNAL(triggered(bool)), this, SLOT(unShadeWindow()));
          }
          else
          {
                    a = menu.addAction(QIcon(style()->standardPixmap(QStyle::SP_TitleBarShadeButton)),tr("Roll up"));
                    a->setEnabled(allow.Shade && !state.Hidden);
                    connect(a, SIGNAL(triggered(bool)), this, SLOT(shadeWindow()));
          }


          // **قوائم وضعية النوافذ ****************************
          menu.addSeparator();

          QMenu* layerMenu = menu.addMenu(tr("&Layer"));

          a = layerMenu->addAction(tr("Always on &top"));
          a->setEnabled(allow.AboveLayer && !state.AboveLayer);
          a->setData(XfitMan::LayerAbove);
          connect(a, SIGNAL(triggered(bool)), this, SLOT(setApplicationLayer()));

          a = layerMenu->addAction(tr("&Normal"));
          a->setEnabled(state.AboveLayer || state.BelowLayer);
          a->setData( XfitMan::LayerNormal);
          connect(a, SIGNAL(triggered(bool)), this, SLOT(setApplicationLayer()));

          a = layerMenu->addAction(tr("Always on &bottom"));
          a->setEnabled(allow.BelowLayer && !state.BelowLayer);
          a->setData(XfitMan::LayerBelow);
          connect(a, SIGNAL(triggered(bool)), this, SLOT(setApplicationLayer()));
      //    qDebug()<<"her";

          // ** امر اغلاق النافذة *****************************
          menu.addSeparator();
          a = menu.addAction(QIcon::fromTheme("process-stop",QIcon(style()->standardPixmap(QStyle::SP_TitleBarCloseButton))), tr("&Close"));
          connect(a, SIGNAL(triggered(bool)), this, SLOT(closeApplication()));

          // ** امر اضافة المشغل الى نافذة الارساء *****************************
           a = menu.addSeparator();
            menu.addAction(m_actParent);
       // ** قوائم انافذة الرئيسية *****************************

       QMenu *mnuDock=menu.addMenu(tr("Dock bar"));
        mnuDock->addActions(m_listQctions);

       menu.exec(event->globalPos());
}
//**********************************عمليات النوافذ***************************
void DActionTaskbar::mousePressEvent(QMouseEvent *event)
{
          if (event->button() == Qt::LeftButton)
          {
                    if (this->isChecked()) {
                              minimizeWindow();
                    }else {
                              restoreWindow();
                    }
          }
          //    else if (mCloseOnMiddleClick && event->button() == Qt::MidButton)
          //    {
          //        closeApplication();
          //    }
}
//****************تصغير النافذة*********************
void DActionTaskbar::minimizeWindow()
{
          xfitMan().minimizeWindow(m_Window);
}
//****************استعادة النافذة*********************
void DActionTaskbar::restoreWindow()
{
          int winDesktop = xfitMan().getWindowDesktop(m_Window);
          if (xfitMan().getActiveDesktop() != winDesktop)
                    xfitMan().setActiveDesktop(winDesktop);
          xfitMan().raiseWindow(m_Window);
}
//************نقل النافذة الى سطح مكتب****************
void DActionTaskbar::moveWindowToDesktop()
{
          QAction* act = qobject_cast<QAction*>(sender());
          if (!act)
                    return;

          bool ok;
          int desk = act->data().toInt(&ok);

          if (!ok)
                    return;

          xfitMan().moveWindowToDesktop(m_Window, desk);
}
//************استعادة النافذة من التكبير****************
void DActionTaskbar::unMaximizeWindow()
{
          xfitMan().deMaximizeWindow(m_Window);
}
//************تحجيم النافذة اي نظليلها****************
void DActionTaskbar::shadeWindow()
{
          xfitMan().shadeWindow(m_Window, true);
}
//************استعادة النافذة من التحجيم****************
void DActionTaskbar::unShadeWindow()
{
          xfitMan().shadeWindow(m_Window, false);
}
//******************غلق النافذة**********************
void DActionTaskbar::closeApplication()
{
          xfitMan().closeWindow(m_Window);
}
//******************تكبير النافذة**********************
void DActionTaskbar::maximizeWindow()
{
          QAction* act = qobject_cast<QAction*>(sender());
          if (!act)
                    return;

          if (act->data().toInt() == XfitMan::MaximizeHoriz)
                    xfitMan().maximizeWindow(m_Window, XfitMan::MaximizeHoriz);

          else if (act->data().toInt() == XfitMan::MaximizeVert)
                    xfitMan().maximizeWindow(m_Window, XfitMan::MaximizeVert);

          else
                    xfitMan().maximizeWindow(m_Window, XfitMan::MaximizeBoth);
}
//******************وضع النافذة فوق او تحت او عادي*************
void DActionTaskbar::setApplicationLayer()
{
          QAction* act = qobject_cast<QAction*>(sender());
          if (!act)
                    return;

          if (act->data().toInt() == XfitMan::LayerAbove)
                    xfitMan().setWindowLayer(m_Window, XfitMan::LayerAbove);

          else if (act->data().toInt() == XfitMan::LayerBelow)
                    xfitMan().setWindowLayer(m_Window, XfitMan::LayerBelow);

          else
                    xfitMan().setWindowLayer(m_Window, XfitMan::LayerNormal);
}

//********************
void DActionTaskbar::setParentActions(QList<QAction *>list)
{
          m_listQctions=list;
}
void DActionTaskbar::setParentAction(QAction *act)
{
          m_actParent=act;
}
//**************

