/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */



#include "quicklaunchaction.h"
#include "dockbarwidget.h"
#include "../libraries/mimetypexdg.h"
#include <QtDebug>
#include <QtCore/QProcess>
#include <QtCore/QUrl>
#include <QtGui/QMessageBox>
#include <QtGui/QDesktopServices>
#include <QtGui/QFileIconProvider>
#include <QSettings>
#include <QTextCodec>
#include <QApplication>
#include <QLocale>
//#include <qtxdg/xdgdesktopfile.h>
//#include <qtxdg/xdgicon.h>
//#include <qtxdg/xdgmime.h>


QuickLaunchAction::QuickLaunchAction(const QString & name,
                                     const QString & exec,
                                     QWidget * parent)
    : QAction(name, parent),
      m_valid(true)
{

    m_settingsMap["name"] = name;
    m_settingsMap["exec"] = exec;


    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));
    QString AppS=MimeTypeXdg::getInfoFromUnknowSufix(exec);
    if(AppS!="unknown"){
        //        QString ic="application-x-shellscript";
        //        if(AppS.contains("python"))
        //            ic="text-x-python";
        QString icon= MimeTypeXdg::getMimeTypeByFile(exec);
        setData(AppS +" \""+exec+"\"");
      //  qDebug( )<<AppS +" "+exec;

        //QIcon iconF=QIcon::fromTheme(icon,QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));
        QIcon  iconF=MimeTypeXdg::getIconFromeTheme(icon);
        setIcon(QIcon(iconF));
        m_type =   ActionScript;

        return;
    }

    setData(exec);
    m_type = ActionLegacy;
    QString App=MimeTypeXdg::getInfoFromDesktopApp(exec);
    if(App!=""){
        setText(App.section("|",0,0));
      //  QIcon iconF=QIcon::fromTheme(App.section("|",1,1),QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));
       QIcon  iconF=MimeTypeXdg::getIconFromeTheme(App.section("|",1,1));
        setIcon(iconF);

    }else{
        setIcon(QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));

    }

}

QuickLaunchAction::QuickLaunchAction(const QString & name,
                                     const QString & exec,
                                     const QString & icon,
                                     QWidget * parent)
    : QAction(parent),
      m_valid(true)
{


   // qDebug()<<"icon----------------------"<<icon;
    int i=name.section("|",1,1).toInt();
    if(i==0){ m_type =  ActionLegacy;}
    if(i==1){ m_type =  ActionScript;}
    if(i==2){ m_type =  ActionFile;}
//    QIcon iconF;
//    if(QFile::exists(icon)){
//       iconF=QIcon(icon);

//    }else{
//      // iconF=QIcon::fromTheme(icon,QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));

//       iconF=QIcon::fromTheme(icon,MimeTypeXdg::getIconFromCharPixmap(QDir("/usr/share/pixmaps/"),icon));
//       if(iconF.isNull())
//           iconF=QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png"));

//    }


    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));



  //qDebug()<<"iconF----------------------"<<iconF.name();
    setText(name.section("|",0,0));
    setData(exec);
//    QPixmap pixmap = iconF.pixmap(QSize(128, 128),QIcon::Active,QIcon::Off);
//    setIcon(QIcon(pixmap));
    // setIcon(iconF);

    m_settingsMap["name"] = name;
    m_settingsMap["exec"] = exec;
    m_settingsMap["icon"] = icon;
}

QuickLaunchAction::QuickLaunchAction(const QString & fileName, QWidget * parent)
    : QAction(parent),
      m_valid(true)
{

    DockBarWidget *quick = qobject_cast<DockBarWidget*>(parent);
    QString lC=quick->locale().name();
    lC= lC.section("_",0,0);
    /******************************************************/
    QFileInfo fi(fileName);
    QString iconXdg= MimeTypeXdg::getMimeTypeByFile(fileName).trimmed();
    QString exec=fileName;

    QIcon iconF;
    if(fi.suffix()=="desktop"){                         /*      .desktop           */
     //   qDebug()<<"this is desktop----: "<<fileName;
        m_type = ActionLegacy;
        QString line=  MimeTypeXdg:: getInfoFromDesktopFile(exec, lC );
        iconXdg=line.section("|",1,1);
        setText(line.section("|",0,0));
     //   iconF=QIcon::fromTheme(iconXdg,QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));

        iconF=QIcon::fromTheme(iconXdg,MimeTypeXdg::getIconFromCharPixmap(QDir("/usr/share/pixmaps/"),iconXdg));
        if(iconF.isNull())
            iconF=QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png"));

        exec=line.section("|",2,2);
        setData(exec);


    }else if( fi.isExecutable() && !fi.isDir()){       /*    exec and script    */

        QString AppS=MimeTypeXdg::getInfoFromUnknowSufix(exec);

        if(AppS!="unknown"){           //script
            setText(fi.fileName());
            setData(AppS +" \""+exec+"\"");
            exec=AppS +" \""+exec+"\"";
            setData(exec);
            iconF=(QIcon::fromTheme(iconXdg,QIcon::fromTheme("application-x-shellscript")));
            m_type =   ActionScript;
        }else{                         //exec
            QString App=MimeTypeXdg::getInfoFromDesktopApp(exec);
            if(App!=""){
                setText(App.section("|",0,0));
                iconXdg=App.section("|",1,1);
            }else{
                 setText(fi.fileName());
            }
  iconF=MimeTypeXdg::getIconFromeTheme(iconXdg);
            setData(exec);
            m_type = ActionLegacy;
        }

    }else if(fi.isDir()){                                     /*       directory          */

        m_type = ActionFile;
        setText(fi.fileName());
        setData(exec);
        QFileIconProvider ip;
        iconXdg=ip.icon(fi).name();
        iconF=(QIcon::fromTheme(iconXdg,QIcon::fromTheme("folder")));

    }else{                                                   /*        other files       */

        m_type = ActionFile;
        setText(fi.fileName());
        setData(exec);
         iconF=MimeTypeXdg::getIconFromeTheme(iconXdg);
    }

 setIcon(iconF);
    //   setIcon(iconF);
//    qDebug()<<"iconXdg----: "<<iconXdg;
//    qDebug()<<"exec-------: "<<exec;
//    qDebug()<<"Name-------: "<<fi.fileName();
//    qDebug()<<"type-------: "<<m_type;

    m_settingsMap["name"] = text()+"|"+QString::number(m_type);
    m_settingsMap["exec"] = exec;
    m_settingsMap["icon"] = iconXdg;
    connect(this, SIGNAL(triggered()), this, SLOT(execAction()));
}

void QuickLaunchAction::execAction()
{
    QString exec(data().toString().section("|",0,0));
    QProcess process;

    QFileInfo fi(exec);
    QStringList list;
    switch (m_type)
    {
    case ActionLegacy:
   //     qDebug()<<"ActionLegacy : "<<fi.path();
        if (fi.path()!="."){
            QString path=fi.path();
            QString name=fi.fileName().section(" ",0,0);
            QString arg=fi.fileName().remove(name);
            exec="\""+path+"/"+name+"\""+arg;

            exec.replace("\"\"","\"");
       //     qDebug()<<"exec1 : "<<exec;
            process.startDetached(exec);

        }else{

            QString name=exec.section(" ",0,0).trimmed();
            QString arg=exec.remove(name).trimmed();

        //    qDebug()<<" arg : "<<arg;
            if(arg.isEmpty()){
                list.clear();

                process.startDetached(name.trimmed(),list,QDir::homePath());
            }else{
                list.append(arg);
                process.startDetached(name.trimmed(),list,QDir::homePath());
              //  qDebug()<<"exec list arg : "<<name.trimmed();
            }


        }
        // process.startDetached(exec);

        break;
    case ActionScript:

      //  qDebug()<<"ActionScript : "<<exec;
        break;

    case ActionFile:

        if (fi.isDir()){

            QSettings setting("elokab","elokabsettings");
            setting.beginGroup("DefaultBrowser");
            bool defaultBrowser=(setting.value("defaultBrowser",true).toBool());

            if(defaultBrowser==true)     {
                //تشغيل مجلدات المستخدم كمجلد المستندات والصور وغيرها

                QDesktopServices::openUrl( QUrl::fromLocalFile(exec) );

            }else{

                QString costumBrowser=(setting.value("costumBrowser","dolphin").toString());
                QProcess process;
                process.startDetached(costumBrowser,QStringList()<<exec);

            }
            setting.endGroup();
        }else{
            QDesktopServices::openUrl( QUrl::fromLocalFile(exec) );
        }
        break;
    }
}
void QuickLaunchAction::setSettingMap(QString txt)
{
   // qDebug()<<"setSettingMap"<<txt;
    QString name=txt.section("|",0,0);
    name.replace("#","|");
    m_settingsMap["name"] =name;
    m_settingsMap["exec"] = txt.section("|",1,1);
    m_settingsMap["icon"] = txt.section("|",2,2);
    setText(name.section("|",0,0));
    setData(txt.section("|",1,1));
    QIcon iconF;
    if(QFile::exists(txt.section("|",2,2))){
        iconF=QIcon(txt.section("|",2,2));
    }else{
     //   iconF=QIcon::fromTheme(txt.section("|",2,2),QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));
      iconF=MimeTypeXdg::getIconFromeTheme(txt.section("|",2,2));
    }
    setIcon(iconF);
}
void QuickLaunchAction::refreshIcon()
{

     QIcon iconF;
     if(QFile::exists(m_settingsMap["icon"])){
        iconF=QIcon(m_settingsMap["icon"]);

     }else{

  iconF=MimeTypeXdg::getIconsFromeTheme(m_settingsMap["icon"],"application-x-executable");

     }
     // setIcon(iconF);
       setIcon(QPixmap(iconF.pixmap(128,128)));
      /*
      //  iconF=QIcon::fromTheme(m_settingsMap["icon"],QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png")));

        iconF=QIcon::fromTheme(m_settingsMap["icon"],MimeTypeXdg::getIconFromCharPixmap(QDir("/usr/share/pixmaps/"),m_settingsMap["icon"]));
        if(iconF.isNull())
            iconF=QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png"));


     }
  //   iconF=QIcon::fromTheme(m_settingsMap["icon"],QIcon::fromTheme("application-x-executable"));

    QPixmap pixmap = iconF.pixmap(QSize(48, 48),QIcon::Active,QIcon::Off);


    setIcon(QIcon(pixmap));*/
}
