/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include <QtGui/QAction>
#include <QtGui/QMenu>
#include <QtGui/QStylePainter>
#include <QtGui/QStyleOptionToolButton>
#include <QtGui/QMouseEvent>
#include <QtGui/QApplication>
#include <QDebug>
#include "dialogaddaction.h"
#include "quicklaunchbutton.h"
#include "dockbarwidget.h"
#include <QToolTip>
#include <QApplication>
#include <QDesktopWidget>
#define MIMETYPE "ddock"


QuickLaunchButton::QuickLaunchButton(QSize size,int id, QuickLaunchAction * act, QWidget * parent)
          : QToolButton(parent),
            m_act(act),
            m_id(id)
{
    setAcceptDrops(true);
    m_size=size;
    setDefaultAction(m_act);
    m_act->setParent(this);
    //    setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    setIconSize(m_size);
    //    setMaximumSize(m_size);
    //    setMinimumSize(m_size);

    m_propertyAct=new QAction(QIcon::fromTheme("document-properties"), trUtf8("properties"), this);
    connect(m_propertyAct, SIGNAL(triggered()), this, SLOT(getProperty()));


    m_deleteAct = new QAction(QIcon::fromTheme("dialog-close",QIcon::fromTheme("list-remove")), trUtf8("Remove from quicklaunch"), this);
    connect(m_deleteAct, SIGNAL(triggered()), this, SLOT(selfRemove()));
    DockBarWidget *quick = qobject_cast<DockBarWidget*>(parent);

    //addAction(m_deleteAct);
    m_menu = new QMenu;

    m_menu->addAction(m_act);
    m_menu->addAction(m_propertyAct);

    m_menu->addSeparator();
    m_menu->addAction(m_deleteAct);
    m_menu->addSeparator();
    QMenu *mnuDock=new QMenu(tr("Dock Bar"),this);

    mnuDock->addAction(quick->actAddActions);
    mnuDock->addAction(quick->actSettings);
    mnuDock->addSeparator();
    mnuDock->addAction(quick->actQuit);
    m_menu->addMenu(mnuDock);
quick->actSettings->setIcon(QIcon::fromTheme("configure",QIcon::fromTheme("document-properties")));
quick->actAddActions->setIcon(QIcon::fromTheme("list-add"));
quick->actQuit->setIcon(QIcon::fromTheme("application-exit"));
    setContextMenuPolicy(Qt::CustomContextMenu);
    connect(this, SIGNAL(customContextMenuRequested(const QPoint&)),
            this, SLOT(this_customContextMenuRequested(const QPoint&)));
    m_anim=new Animmation(this);
    connect(this, SIGNAL(clicked()), this, SLOT(animmation()));
}

QuickLaunchButton::~QuickLaunchButton()
{
          //m_act->deleteLater();
}

void QuickLaunchButton::setSize(QSize size)
{
          m_size=size;
          setIconSize(m_size);

}
void QuickLaunchButton::setAnimation(int animation)
{
  m_anim->setStyle(animation);
  m_act->refreshIcon();
}
QHash<QString,QString> QuickLaunchButton::settingsMap()
{
          Q_ASSERT(m_act);
          return m_act->settingsMap();
}


void QuickLaunchButton::this_customContextMenuRequested(const QPoint & pos)
{

          m_menu->popup(mapToGlobal(pos));
}

void QuickLaunchButton::selfRemove()
{
          emit buttonDeleted(m_id);
}



void QuickLaunchButton::mousePressEvent(QMouseEvent *e)
{


          if (e->button() == Qt::LeftButton && e->modifiers() == Qt::ControlModifier)
          {

                    m_dragStart = e->pos();
                    return;
          }

          QToolButton::mousePressEvent(e);
}

void QuickLaunchButton::mouseMoveEvent(QMouseEvent *e)
{
     //     QToolTip::showText ( QPoint (this->pos().x(),this->pos().y()-10), this->text() );
    if (!(e->buttons() & Qt::LeftButton))
    {
              return;
    }

    if ((e->pos() - m_dragStart).manhattanLength() < QApplication::startDragDistance())
    {
              return;
    }

    if (e->modifiers() != Qt::ControlModifier)
    {
              return;
    }

    QDrag *drag = new QDrag(this);
    QMimeData *mimeData = new QMimeData();
    QByteArray ba;
    ba.setNum(m_id);
    mimeData->setData(MIMETYPE, ba);
    drag->setMimeData(mimeData);
//drag->setPixmap(this->icon().pixmap(iconSize()));

    drag->exec();

    // Icon was droped outside the panel, remove button

    if (!drag->target())
    {
       selfRemove();
    }
}

void QuickLaunchButton::dragMoveEvent(QDragMoveEvent * e)
{

    if (e->mimeData()->hasFormat(MIMETYPE))
              e->acceptProposedAction();
    else
              e->ignore();
}

void QuickLaunchButton::dragEnterEvent(QDragEnterEvent *e)
{
    if (e->mimeData()->hasFormat(MIMETYPE))
    {
              int other = e->mimeData()->data(MIMETYPE).toInt();
              // do not replace self with self
              if (other == m_id)
                        return;

              emit switchButtons(m_id, other);
    }
}

void QuickLaunchButton::getProperty()
{
  QRect screen = QApplication::desktop()->screenGeometry(-1);

          DialogAddAction *dlg=new DialogAddAction(m_act);
          connect(dlg,SIGNAL(accepteChange(QString)) ,this,SLOT(changeAction(QString)));
          dlg->move(screen.width()/2-dlg->width()/2,screen.height()/2-dlg->height()/2);
          dlg->show();

}

void QuickLaunchButton::changeAction(QString txt)
{

          m_act->setSettingMap(txt);
          emit settingsChanged();
}

//void QuickLaunchButton::enterEvent(QEvent *)
//{

//    //  m_anim->AnimationStart(m_act,this->iconSize());
//}

//void QuickLaunchButton::leaveEvent(QEvent *)
//{

//     //  m_anim->AnimationStop();
//}
void QuickLaunchButton::animmation()
{
    m_anim->AnimationStart(m_act,this->iconSize());
}

