/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include "elokabsettings.h"
#include <QIcon>
#include <QDir>
#include <QDebug>
#include <QCoreApplication>
elokabSettings::elokabSettings(const QString &organization, const QString &application, QObject *parent) :
        QSettings(organization,application, parent)
{

    m_fileWatcher=new QFileSystemWatcher;
    m_fileWatcher->addPath(this->fileName());
    connect(m_fileWatcher, SIGNAL(fileChanged(QString)), this, SLOT(fileChanged()));

 //qDebug()<<this->fileName();
}
elokabSettings::elokabSettings( QObject *parent) :
    QSettings("elokab","shortcut", parent)
{
    m_fileWatcher=new QFileSystemWatcher;
    m_fileWatcher->addPath(this->fileName());
    connect(m_fileWatcher, SIGNAL(fileChanged(QString)), this, SLOT(shortcutsCanged()));

 //qDebug()<<this->fileName();
}
void elokabSettings::fileChanged()
{
    sync();
    QString iconThem=loadIconThems();
    if(iconThem!=m_iconTheme){
        m_iconTheme=iconThem;
       emit iconThemeChanged();
    }
    QString styleTheme=loadStyleWidget();
    if(m_styleWidget!=styleTheme){
        m_styleWidget=styleTheme;
        emit widgetStyleChanged();
    }
    QString Theme=loadtheme();
    if(m_Themes!=Theme){
        m_Themes=Theme;
        emit ThemesChanged();
        qDebug()<<"settingsChanged theme";
    }
}
void elokabSettings::shortcutsCanged()
{
    sync();
    emit shortcutsKeysChanged();
}

QString elokabSettings::loadIconThems()
{

    this->beginGroup("Themes");
    QString icnThem=  this->value("iconsTheme",QIcon::themeName()).toString();
     this->endGroup();
    if(icnThem=="hicolor"||icnThem.isEmpty()){

        QStringList failback;
        failback << "oxygen"<< "Mint-X"<< "Humanity"<< "Tango"<< "Prudence-icon"<< "elementary"<< "gnome";

        QDir dir("/usr/share/icons/");
        foreach (QString s, failback)
        {
            if (dir.exists(s))
            {
                icnThem=s;
             //   setValue("iconsTheme",icnThem);
                break;
            }
        }
    }
    if(icnThem.isEmpty())
            icnThem="oxygen";

    return icnThem;
}

QString elokabSettings::loadStyleWidget()
{
  this->beginGroup("Style");
    QString style=this->value("Name",trUtf8("default")).toString();
  this->endGroup();
    return style;
}

QString elokabSettings::loadtheme()
{
         sync();
  this->beginGroup("Themes");
    QString Theme=this->value("ThemeName",trUtf8("default")).toString();
  this->endGroup();
    qDebug()<<"theme name :"<<Theme;
    return Theme;
}
