/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include "menugen.h"

MenuGen::MenuGen(const QString &locale, QObject *parent) :
        QObject(parent), m_locale(locale)
{
m_sWatcher=new QFileSystemWatcher;
        connect(m_sWatcher, SIGNAL(directoryChanged(QString)), this, SLOT(genirateAppFile()));
            loadSettings();
            saveSettings();
}

bool MenuGen::appFilesExist()
{

          QFile fileApp( QDir::homePath()+"/.config/elokab/applictions");

          QString dirUsers="/usr/share/applications";
          QString dirLocal=  QDir::homePath()+"/.local/share/applications";

          int count1=countDesktopDir(dirLocal);             /*ex:  /home/usrer/.local/share/applications  */
          int count2=countDesktopDir(dirUsers);             /*     "/usr/share/applications"                         */
          int count=count1+count2;                                    /*    عدد التطبيقات في المسارين                                  */

          if (count!=m_appCount||!fileApp.exists()){         /*  اذا كان العدد مختلف او القائمة غير موجودة                  */
                    genirateAppFile();
                    m_appCount=count;
                    saveSettings();

                    return true;
          }
          return false;
}

//! [0-1] ------------------------------------------------حمل عدد التطبيقات  ---------------------------------------------- */
void MenuGen::loadSettings()
{
    QSettings setting("elokab","elokabsettings");
          setting.beginGroup("MenuAppCount");
          m_appCount=setting.value("Num",0).toInt();
          setting.endGroup();
}
//! [0-2] ------------------------------------------------حفظ عدد التطبيقات  ---------------------------------------------- */
void MenuGen::saveSettings()
{
          QSettings setting("elokab","elokabsettings");
          setting.beginGroup("MenuAppCount");
          setting.setValue("Num",m_appCount);
          setting.endGroup();
       //     emit appDesktopChanged();
}
//! [1-1] ------------------------------------------------عد المجلدات  ---------------------------------------------- */
int MenuGen::countDesktopDir(QString dir)
{
          int count=0;
          count=count+ countDesktopFile(dir);

          QString subdir;
          QDir dirS(dir);
          foreach ( subdir, dirS.entryList(QDir::AllDirs| QDir::NoSymLinks| QDir::NoDotAndDotDot )){
                    count=count+ countDesktopFile(dir+"/"+subdir);

          }
          return count;
}
//! [1-1] ------------------------------------------------عد الملفات  ---------------------------------------------- */
int MenuGen::countDesktopFile(QString   dir)
{
          int count=0;
          QString subfile;
          QDir dirS(dir);
          foreach ( subfile, dirS.entryList(QDir::AllEntries|  QDir::NoDotAndDotDot )){
                    QFileInfo fi(dir+"/"+subfile);
                    if(!fi.isDir()&&fi.completeSuffix()=="desktop"){
                              count++;
                    }
          }
          return count;
}
//! [2] ------------------------------------------------انشاء ملف بالتطبيقات الموجودة  ------------------------------------- */
void MenuGen::genirateAppFile()
{


          QFile fileApp( QDir::homePath()+"/.config/elokab/applictions");
         QFile fileMim( QDir::homePath()+"/.config/elokab/mimeApp");
          if (!fileApp.open(QFile::WriteOnly)){
                    return ;
          }
          fileMim.open(QFile::WriteOnly);
          QTextStream out(&fileApp);
           QTextStream outMim(&fileMim);
          out.setCodec(QTextCodec::codecForName("UTF-8"));
          outMim.setCodec(QTextCodec::codecForName("UTF-8"));
          QString dirUsers="/usr/share/applications";
          QString dirLocal=  QDir::homePath()+"/.local/share/applications";
          chargeDesktopDir( dirUsers);
          chargeDesktopDir( dirLocal);


//           for (int i=0;i<wordList.count();i++){
//                    QString data=wordList.at(i);
//                    out << data+"\n";
//          }
           foreach (QString s, wordList) {
                  out << s+"\n";
           }
           foreach (QString s, mimList) {
                  outMim << s+"\n";
           }
          m_appCount=wordList.count();
          saveSettings();
          wordList.clear();
          mimList.clear();
          fileApp.close();
          fileMim.close();
//emit appDesktopChanged();
}
//! [2-1] ------------------------------------------------تحميل المجادات المحددة  ------------------------------------- */
void MenuGen::chargeDesktopDir(QString dir)
{

          chargeDesktopFile(dir);


          QString subdir;
          QDir dirS(dir);
          foreach ( subdir, dirS.entryList(QDir::AllDirs| QDir::NoSymLinks| QDir::NoDotAndDotDot )){
                    chargeDesktopFile(dir+"/"+subdir);

          }
          //return true;
}
//! [2-2] ------------------------------------------------تحميل الملفات المحددة  ------------------------------------- */
void MenuGen::chargeDesktopFile(QString   dir)
{
          QString subfile;
          QDir dirS(dir);
          if (!m_sWatcher->directories().contains(dir))
              m_sWatcher->addPath(dir);
            qDebug()<<"mWatcher:--------: "<<dir;
          foreach ( subfile, dirS.entryList(QDir::AllEntries|  QDir::NoDotAndDotDot )){
                    QFileInfo fi(dir+"/"+subfile);
                    if(!fi.isDir()&&fi.completeSuffix()=="desktop"){
                              QString data=openDesktopEntry(dir+"/"+subfile);
                              if(!data.isEmpty())
                                        wordList.append(data);

                    }
          }
}
//! [2-3] ---------------------------------------فتح الملفات لاستحراج المعلومات  ------------------------------------- */
QString MenuGen::openDesktopEntry(QString file)
{

          QFileInfo fi(file);
          QString baseName=fi.baseName();
          QSettings settings(file,QSettings::IniFormat);
          settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
          settings.beginGroup("Desktop Entry");
          QString nameold=settings.value("Name",baseName).toString();
          QString name=settings.value("Name["+m_locale+"]",nameold).toString();
          QString exec=settings.value("Exec",baseName).toString();
          QString TryExec=settings.value("TryExec",exec).toString();
          QString icon=settings.value("Icon",baseName).toString();
          bool NoDisplay=settings.value("NoDisplay",false).toBool();
          //***********************mime------------


//     foreach (QString s, mimtype) {
//             if(!s.isEmpty())
//                     mimList.append(s+"|"+file);
//     }




          settings.endGroup();
          if(NoDisplay==true)return "";
          QString category="Other";
          QString mime;
          QFile files(file);
          if(files.open( QFile::ReadOnly)){
                    QTextStream textStream(&files);
                    QString line = textStream.readLine();//premier line;

                    while   (!line.isNull()) {
                              line = textStream.readLine();
                              if(line.contains("Categories=")){
                                        category=line.remove("Categories=");
                                      //  break;
                              }
                              if(line.contains("MimeType=")){
                                        mime=line.remove("MimeType=");
                                       // break;
                              }
                    }
          }
          files.close();
          if(!mime.isEmpty()){
                  QStringList list=mime.split(";",QString::SkipEmptyParts);
                  foreach (QString s, list) {
                            mimList.append(s+"|"+TryExec);
                  }


          }



          TryExec.remove("-caption");
          return name+"|"+TryExec.section("%",0,0)+"|"+category+"|"+icon+"|"+file;

}
