/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)GPL3+
 *
 * elokab - a lightweight, Qt based, desktop toolset
 * https://sourceforge.net/project/elokab/
 *
 * Copyright: 2013-2014 elokab team
 * Authors:
 *   Abou Zakaria <yahiaui@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */


#include "mimetypexdg.h"
#include <QFile>
#include <QTextStream>
#include <QFileInfo>
#include <QDebug>
#include <QSettings>
#include <QDir>
#include <QTextCodec>
#include <QProcess>
#include <QIcon>
#include <QFileInfo>
MimeTypeXdg::MimeTypeXdg()

{

}
QString MimeTypeXdg::getMimeTypeByFile(QString fileName)
{

          QProcess process;
          process.start(QString("xdg-mime query filetype %1").arg("\""+fileName+"\""));
          process.waitForFinished();
          QString result= process.readAll();

          if(result.isEmpty()){
                    QFileInfo fi(fileName);
                  //  qDebug()<<"by sufix";
                    return  getMimeTypeBySufix(fi.suffix());
          }
          result=result.section(";",0,0); // value "application/x-unknow"


          return iconName(result);
}
QString MimeTypeXdg::getMimeType(QString fileName)
{
        QProcess process;
        process.start(QString("xdg-mime query filetype %1").arg("\""+fileName+"\""));
        process.waitForFinished();
        QString result= process.readAll();
        qDebug()<<"by mimetype "<<result;
                  return result;
}

QString MimeTypeXdg::getAppByMimeType(QString fileName)
{
    QString mime=    getMimeType( fileName);
    QFile file(QDir::homePath()+"/.config/elokab/mimeApp");

    if (!file.open(QFile::ReadOnly)){
              return "";
    }
    QTextStream textStream(&file);
    QString line;

    line= textStream.readLine();//premier line
    while (!line.isNull()) {
              qDebug()<< line.section("|",0,0);
            if(line.section("|",0,0).trimmed()==mime.trimmed()){
                    qDebug()<<"this app is :"<< line.section("|",1,1);
                    return  line.section("|",1,1);
            }
            line= textStream.readLine();
  }
  file.close();
    return "";
}
QStringList MimeTypeXdg::getAppSByMimeType(QString fileName)
{
    QString mime=    getMimeType( fileName);
    QFile file(QDir::homePath()+"/.config/elokab/mimeApp");
QStringList list;
    if (!file.open(QFile::ReadOnly)){
              return list;
    }
    QTextStream textStream(&file);
    QString line;

    line= textStream.readLine();//premier line
    while (!line.isNull()) {
              qDebug()<< line.section("|",0,0);
            if(line.section("|",0,0).trimmed()==mime.trimmed()){
                    qDebug()<<"this app is :"<< line.section("|",1,1);
                    list.append(  line.section("|",1,1));
            }
            line= textStream.readLine();
  }
  file.close();
 return list;
}
QString MimeTypeXdg::iconName(QString result )
{

          QString m_groupType;
          QString m_fileSubType;

          m_groupType=result.section("/",0,0); // value "application"
          m_fileSubType=result.section("/",1,1); // value "x-unknow"

         //      qDebug()<<"process.readAll() : "<< QString("%1-%2").arg(m_groupType, m_fileSubType.trimmed());;
          QStringList names;
          names << QString("%1-%2").arg(m_groupType, m_fileSubType.trimmed());// value "application-unknow"

          names << QString("%1-x-%2").arg(m_groupType, m_fileSubType.trimmed());// value "application-x-unknow"

          names << QString("%1-x-generic").arg(m_groupType);// value "application-x-generic"
          names << QString("%1-generic").arg(m_groupType);// value "application-generic"
          names << QString("%1").arg(m_groupType);// value "application"

          foreach (QString s, names)
          {
                //    qDebug()<<"iconNamel list() -------------: "<<s;
                    if (!QIcon::fromTheme(s).isNull()){
                             // qDebug()<<"iconNamel retined() -------------: "<<s;
                              return s;
                    }
          }

          return "unknown";
}
QString MimeTypeXdg::getMimeTypeBySufix(QString sufix)
{


          // mimeTypeالبحث عن اسم الايقونة حسب اللاحقة
          QFile file("/usr/share/mime/globs");
          if (!file.open(QFile::ReadOnly)){
                    return "unknown";
                //    qDebug()<<"by sufix unknown";
          }
          QTextStream textStream(&file);
          QString line;

          line= textStream.readLine();//premier line
          while (!line.isNull()) {
                    if(line.contains("*."+sufix)){
                           //   qDebug()<<line;             //value "application/x-unknow:*.sufix"
                              line=line.section(":",0,0); // value "application/x-unknow"
                              // line=line.replace("/","-"); // value "application-x-unknow"

                           //   qDebug()<<"by sufix line" <<line;
                              return iconName(line );
                    }
                    /************************************/
                    line = textStream.readLine();

          }
          return "unknown";
}
QString MimeTypeXdg::getIconFillBack(QString mimeType)
{
          QString iconBase=mimeType.section("-",0,0);
          if(iconBase=="text"||iconBase=="video"||iconBase=="image"||iconBase=="audio"){
                //    qDebug()<<"iconBase : "<<iconBase;
                    return iconBase;
          }
          return "unknown";
}
QString MimeTypeXdg::getInfoFromUnknowSufix(QString fileName)
{
          QFile file(fileName);
          if (!file.open(QFile::ReadOnly)){

                    return "unknown";
          }
          QTextStream textStream(&file);
          QString line;

          line= textStream.readLine();//premier line
       //   qDebug()<<"lineScripte : "<<line;
          if(line.contains("#!"))
                    return line.remove("#!").trimmed();


         QSettings settings(fileName,QSettings::IniFormat);
         settings.beginGroup("Desktop Entry");
          if(!settings.contains("Name"))
                  return "Desktop" ;
          settings.endGroup();

          return  "unknown";
}

QString MimeTypeXdg::getInfoFromDesktopFile(QString fileName,QString lC )
{
          QFileInfo fi(fileName);
          QString baseName=fi.baseName();

          QSettings settings(fileName,QSettings::IniFormat);
          settings.setIniCodec(QTextCodec::codecForName("UTF-8"));
          settings.beginGroup("Desktop Entry");
          QString nameold=settings.value("Name",baseName).toString();
          QString name=settings.value("Name["+lC+"]",nameold).toString();
          QString exec=settings.value("Exec",baseName).toString();

          QString Commentold=settings.value("Comment",baseName).toString();
          QString Comment=settings.value("Comment["+lC+"]",Commentold).toString();

 if(name.isEmpty())name=baseName;
          QString icon=settings.value("Icon",baseName).toString();
          settings.endGroup();
exec.remove("%U");
exec.remove("%u");
exec.remove("-caption");
exec.remove("%c");
exec.remove("%i");
exec.remove("%f");
exec.remove("%F");

//qDebug()<<"vlc"<<name+"/"+icon+"/"+exec;
  return name+"|"+icon+"|"+exec;
}

QString MimeTypeXdg::getInfoFromDesktopApp(QString fileName )
{

          QFileInfo fi(fileName);

          QFile file(QDir::homePath()+"/.config/elokab/applictions");

          if (!file.open(QFile::ReadOnly)){
                    return "";
          }
          QTextStream textStream(&file);
          QString line;

          line= textStream.readLine();//premier line
          while (!line.isNull()) {
                //    qDebug()<<line;
                    QString exec=line.section("|",1,1);

                    if(exec.contains(fi.fileName())){
                            //  qDebug()<<"contain :"<<exec;
                              QString name=line.section("|",0,0);
                              QString icon=line.section("|",3,3);
                              file.close();
                              return name+"|"+icon;

                    }
                    line= textStream.readLine();
          }
          file.close();
          return "";
}

QString MimeTypeXdg::getIconFromDesktopApp(QString fileName )
{



          QFile file(QDir::homePath()+"/.config/elokab/applictions");

          if (!file.open(QFile::ReadOnly)){
                    return "";
          }
          QTextStream textStream(&file);
          QString line;

          line= textStream.readLine();//premier line
          while (!line.isNull()) {

                    QString exec=line.section("|",1,1);

                    if(exec.contains(fileName)){
                            //  qDebug()<<"contain :"<<exec;
                            //  QString name=line.section("|",0,0);
                              QString icon=line.section("|",3,3);
                              file.close();
                              return icon;

                    }
                    line= textStream.readLine();
          }
          file.close();
          return "";
}
QIcon  MimeTypeXdg::getIconFromCharPixmap(QDir dir,QString iconName )
{

     QIcon icon(chargeIconFile(dir.path(),iconName));
     if(!icon.isNull()){
       //  qDebug()<<"getIconFromCharPixmap******** :"<<iconName;
           return icon;

     }

    foreach(QString subdir,dir.entryList(QDir::AllDirs| QDir::NoSymLinks| QDir::NoDotAndDotDot )) {

 // qDebug()<<"getIconFromCharPixmap******** :"<<dir.path()+"/"+subdir;
        QIcon icon(getIconFromCharPixmap(dir.path()+"/"+ subdir, iconName));
       if(!icon.isNull())
            return icon;


    }
    return QIcon("");
}
QIcon MimeTypeXdg::chargeIconFile(QString dir,QString iconName)
{

          QDir dirS(dir);
          foreach(QString file,dirS.entryList(QDir::AllEntries| QDir::NoSymLinks| QDir::NoDotAndDotDot )) {
              QFileInfo fi(dirS.path()+"/"+file);
              QString base = fi.baseName();

              if(iconName==base){
// qDebug()<<"getIconFromCharPixmap******** :"<<dirS.path()+"/"+file;
                  return QIcon(dirS.path()+"/"+file);

              }
          }
          return QIcon("");
}

QIcon MimeTypeXdg::getIconFromeTheme(QString  iconName)
{

    //[1]-----------
    QIcon iconF=QIcon::fromTheme(iconName);
 //qDebug()<<"result ; "<<iconName;
    //[2]------------
    if(iconF.isNull()||iconF.isDetached()){
        iconF=getIconFromCharPixmap(QDir("/usr/share/pixmaps/"),iconName);

    }

    //[3]------------fillback
    if(iconF.isNull())
        iconF=QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png"));

    //[4]------------
    QPixmap pix=iconF.pixmap(QSize(128, 128));
    if(pix.isNull())
        iconF=QIcon(":/icons/applications-system.png");

    return iconF.pixmap(QSize(128, 128));
}

QIcon MimeTypeXdg::getIconsFromeTheme(QString  iconName,QString fillback)
{
        QIcon iconF;
        if(QFile::exists(iconName)){
                iconF=QIcon(iconName);
                return iconF;
        }

    //[1]-----------iconName
   iconF=QIcon::fromTheme(iconName);

    //[2]------------pixmaps
    if(iconF.isNull()||iconF.isDetached()){
        iconF=getIconFromCharPixmap(QDir("/usr/share/pixmaps/"),iconName);

    }

    //[3]------------fillback

     if(iconF.isNull())
      iconF=QIcon::fromTheme(fillback);

     //[4]------------fillback from application-x-executable
    if(iconF.isNull())
        iconF=QIcon::fromTheme("application-x-executable",QIcon(":/icons/applications-system.png"));

    //[4]------------fillback from resource
    QPixmap pix=iconF.pixmap(QSize(128, 128));
    if(pix.isNull())
        iconF=QIcon(":/icons/applications-system.png");

    return iconF.pixmap(QSize(128, 128));
}

QString MimeTypeXdg::getinfoAppFromDesktopApp(QString appName ,QString className)
{



          QFile file(QDir::homePath()+"/.config/elokab/applictions");

          if (!file.open(QFile::ReadOnly)){
                    return "";
          }
          QTextStream textStream(&file);
          QString line;

          line= textStream.readLine();//premier line
          while (!line.isNull()) {

                    QString exec=line.section("|",1,1);
                    QString deskt=line.section("|",4,4);
                    if(exec.contains(appName)||exec.contains(className)
                            ||deskt.contains(appName)||deskt.contains(className)){
                            //  qDebug()<<"contain :"<<exec;
                              QString name=line.section("|",0,0);

                              QString icon=line.section("|",3,3);
                              file.close();
                              return name+"|"+exec+"|"+icon;

                    }
                    line= textStream.readLine();
          }
          file.close();
          return "";
}

